# Copyright (c) 2021 Jerome Arbez-Gindre
# frozen_string_literal: true

require('defmastership')

RSpec.describe(DefMastership::RenameIncludedFilesModifier) do
  subject(:modifier) do
    described_class.new(
      'fake config'
    )
  end

  let(:adoc_texts) do
    {
      'file1.adoc' => "file1 line1\nfile1 line2",
      'file2.adoc' => "file2 line1\nfile2 line2"
    }
  end

  describe '.new' do
    it { is_expected.not_to(be(nil)) }
    it { is_expected.to(have_attributes(config: 'fake config')) }
    it { is_expected.to(have_attributes(changes: [])) }
  end

  describe '#do_modifications' do
    let(:line_modifier) { instance_double(DefMastership::RenameIncludedFilesLineModifier, 'lineModifier') }

    before do
      allow(DefMastership::RenameIncludedFilesLineModifier).to(
        receive(:from_config).with('fake config').and_return(line_modifier)
      )
      allow(line_modifier).to(receive(:replace).with("file1 line1\n").and_return("new file1 line1\n"))
      allow(line_modifier).to(receive(:replace).with('file1 line2').and_return('new file1 line2'))
      allow(line_modifier).to(receive(:replace).with("file2 line1\n").and_return("new file2 line1\n"))
      allow(line_modifier).to(receive(:replace).with('file2 line2').and_return('new file2 line2'))
      allow(line_modifier).to(receive(:changes).and_return([%w[from1 to1], %w[from2 to2]]))
      allow(line_modifier).to(receive(:config).and_return('fake config'))
    end

    context 'when detailed expectations' do
      before { modifier.do_modifications(adoc_texts) }

      it do
        expect(DefMastership::RenameIncludedFilesLineModifier).to(
          have_received(:from_config).with('fake config')
        )
      end

      it { expect(line_modifier).to(have_received(:replace).with("file1 line1\n")) }
      it { expect(line_modifier).to(have_received(:replace).with('file1 line2')) }
      it { expect(line_modifier).to(have_received(:replace).with("file2 line1\n")) }
      it { expect(line_modifier).to(have_received(:replace).with('file2 line2')) }
      it { expect(line_modifier).to(have_received(:changes)) }
      it { is_expected.to(have_attributes(config: 'fake config')) }
      it { is_expected.to(have_attributes(changes: [%w[from1 to1], %w[from2 to2]])) }
    end

    it do
      expected_adoc = {
        'file1.adoc' => "new file1 line1\nnew file1 line2",
        'file2.adoc' => "new file2 line1\nnew file2 line2"
      }
      expect(modifier.do_modifications(adoc_texts)).to(eq(expected_adoc))
    end
  end
end
